/*
 * ********************************************************************************************************************
 *
 *                     :*$@@%$*:                         ;:                ;;    ;;
 *                   :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                  :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                  =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                  =@*                                  %!              @ $= % %@=   =%@!      %=
 *             *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *           %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *         ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *         $@*   ;@@@%=!:                *@*
 *         =@$    ;;;!=%@@@@=!           =@!
 *          %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *           ;%@@$=$@@%*       *@@@$=%@@%;
 *              ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.persagy.server.controllers.objects

import com.persagy.server.datacenter.models.entities.dict.DictSpaceProject
import com.persagy.server.datacenter.models.entities.objects.Building
import com.persagy.server.datacenter.models.entities.objects.ZoneSpace
import com.persagy.server.datacenter.models.entities.task.SpaceType
import com.persagy.server.datacenter.models.requests.DCSQueryRequest
import com.persagy.server.datacenter.models.requests.UpDataFloorAndBuildingRequests
import com.persagy.server.datacenter.models.response.SpaceSQueryResponse
import com.persagy.server.services.dict.DictSpaceProjectService
import com.persagy.server.services.objects.ZoneSpaceBaseService
import com.persagy.server.services.relation_calc.MepSystemCalcService
import com.sybotan.database.SFilter
import com.sybotan.service.SPageContext
import com.sybotan.service.models.requests.SCountRequest
import com.sybotan.service.models.requests.SCreateRequest
import com.sybotan.service.models.requests.SUpdateRequest
import com.sybotan.service.models.responses.SBaseResponse
import com.sybotan.service.models.responses.SCountResponse
import com.sybotan.service.models.responses.SCreateResponse
import com.sybotan.service.models.responses.SQueryResponse
import io.swagger.annotations.Api
import io.swagger.annotations.ApiOperation
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

/**
 * 空间父类接口
 *
 * @author 张维新
 */
@Api(tags = ["AF、空间"])
@RestController
@RequestMapping("/object/zone")
open class ZoneSpaceController {
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(ZoneSpaceController::class.java)
    } // Companion object

    @Autowired
    lateinit var mepSystemCalc: MepSystemCalcService

    /**
     * 查询空间信息
     *
     * @param   request     查询信息条件
     * @return              查询结果
     */
//    @ApiOperation(value = "查询空间信息", notes = "级联查询：建筑（building），楼层（floor），空间类型（zoneName）")
//    @PostMapping(value = ["/query"])
//    fun query(@RequestBody request: SQueryRequest): SQueryResponse<ZoneSpace> {
//        logger.debug("metaSpaceQuery =${request.toJson()}")
////        arrayListOf(SFilter.eq("projectId", SPageContext.getHeader("ProjectId").toString()))
//        val projectId = SPageContext.getHeader("ProjectId")
//        if (projectId.isNullOrEmpty()){
//            return ZoneSpaceBaseService.pageQuery(request)
//        }else{
//            return ZoneSpaceBaseService.pageQuery(request, arrayListOf(SFilter.eq("projectId", projectId!!)))
//        }
//    } // Function query()

    /**
     * 根据条件查询统计数量
     */
    @ApiOperation(value = "根据条件查询统计数量", notes = "")
    @PostMapping(value = ["/count"])
    fun count(@RequestBody request: SCountRequest): SCountResponse {
        val withFilterList = ArrayList<SFilter>()
        withFilterList.add(SFilter.eq("projectId", SPageContext.getHeader("ProjectId").toString()))
        return ZoneSpaceBaseService.count(request,withFilterList)
    } // Function count()

    /**
     * 查询空间信息
     */
    @ApiOperation(value = "查询空间信息", notes = "级联查询：建筑（buildingList），楼层（floorlList），空间类型（objectType）未明确楼层(isnull),未明确建筑(isnull)")
    @PostMapping(value = ["/query"])
    fun querySpace(@RequestBody request: DCSQueryRequest): SQueryResponse<ZoneSpace> {
        return ZoneSpaceBaseService.querySpaces(request)
    } // Function querySpace()


    /**
     * 查询业务空间的建筑楼层，包含受影响的空间数量
     */
    @ApiOperation(value = "查询业务空间的建筑楼层，包含受影响的空间数量", notes = "")
    @PostMapping(value = ["/building-floor"])
    fun querySpBuildingFloor(): SQueryResponse<Building>{
        return ZoneSpaceBaseService.spBuildingFloor()
    }

    /**
     * 查询业务空间的所有类型
     */
    @ApiOperation(value = "查询业务空间的所有类型", notes = "级联其它分区（zoneType）")
    @PostMapping(value = ["/type"])
    fun queryZoneTypeSpace(@RequestBody request: DCSQueryRequest): SQueryResponse<DictSpaceProject>{
        return ZoneSpaceBaseService.zoneTypequery(request)
    } // Fun


    /**
     * 创建业务空间类型
     */
    @ApiOperation(value = "创建业务空间类型", notes = "")
    @PostMapping(value = ["/create-type"])
    fun createZoneType(@RequestBody request: SCreateRequest<DictSpaceProject>): SCreateResponse<DictSpaceProject> {
        return DictSpaceProjectService.zoneTypeCreate(request)
    } // Fun

    /**
     * 删除业务空间类型
     */
    @ApiOperation(value = "删除业务空间类型", notes = "")
    @PostMapping(value = ["/delete-type"])
    fun deleteZoneType(@RequestBody idList: ArrayList<DictSpaceProject>): SBaseResponse {
        return DictSpaceProjectService.zoneTypeDelete(idList)
    } // Fun


    /**
     * 创建空间分区
     *
     * @param   request     创建空间分区
     * @return                  创建结果信息
     */
    @ApiOperation(value = "创建空间分区对象", notes = "必填：空间类型（ObjectType）")
    @PostMapping(value = ["/create"])
    fun createSpace(@RequestBody request: SCreateRequest<ZoneSpace>): SCreateResponse<ZoneSpace> {
        return ZoneSpaceBaseService.createList(request)
    } // Function create()

    /**
     * 根据id删除分区
     *
     * @param   idList      id数组
     * @return              删除的结果信息
     */
    @ApiOperation(value = "根据id删除分区对象", notes = "")
    @PostMapping(value = ["/delete"])
    fun delete(@RequestBody idList: ArrayList<ZoneSpace>): SBaseResponse {
        return ZoneSpaceBaseService.deleteByKeysList(idList)
    } // Function delete()

    /**
     * 更新分区信息
     *
     * @param   request     更新的内容对象
     * @return              更新的结果
     */
    @ApiOperation(value = "更新分区对象信息", notes = "")
    @PostMapping(value = ["/update"])
    fun update(@RequestBody request: SUpdateRequest<ZoneSpace>): SBaseResponse {
        return ZoneSpaceBaseService.updateList(request)
    } // Function update()

    /**
     * 更新分区信息 Outline
     *
     * @param   request     更新的内容对象
     * @return              更新的结果
     */
    @ApiOperation(value = "更新分区对象信息-Outline", notes = "")
    @PostMapping(value = ["/update-outline"])
    fun updateOutline(@RequestBody request: SUpdateRequest<ZoneSpace>): SBaseResponse {
        return ZoneSpaceBaseService.upDateOutline(request)
    } // Function update()

    /**
     * 查询空间信息
     *
     * @param   request     查询信息条件
     * @return              查询结果
     */
    @ApiOperation(value = "查询空间信息-outline", notes = "级联查询：建筑（building），楼层（floor），空间类型（objectType）")
    @PostMapping(value = ["/query-outline"])
    fun queryOutline(@RequestBody request: DCSQueryRequest): SpaceSQueryResponse {
//        logger.debug("metaSpaceQuery =${request.toJson()}")
//        arrayListOf(SFilter.eq("projectId", SPageContext.getHeader("ProjectId").toString()))
        val projectId = SPageContext.getHeader("ProjectId")
        if (!request.projection.isNullOrEmpty()){
            request.projection!!.clear()
        }
        val list = ArrayList<String>()
        list.add("Outline")
        list.add("RoomID")
        list.add("RoomName")
        list.add("RoomLocalID")
        list.add("RoomLocalName")
        list.add("ObjectType")
        list.add("BuildingId")
        list.add("FloorId")
        list.add("State")
        request.projection = list
        return ZoneSpaceBaseService.querySpaces(request)

    } // Function query()

    /**
     * 修改空间和楼层建筑的关系
     *
     * @param   upDataFloorAndBuildingRequestsList     修改对象列表
     */
    @ApiOperation(value = "修改业务空间对象和建筑楼层的关系", notes = "spaceId 业务空间id,id 楼层或建筑id,type  业务空间的类型")
    @PostMapping(value = ["/update-fl-bd"])
    fun  spFloorAndBuilding(@RequestBody upDataFloorAndBuildingRequestsList:ArrayList<UpDataFloorAndBuildingRequests>): SBaseResponse {
        return ZoneSpaceBaseService.upDataFloorORBuilding(upDataFloorAndBuildingRequestsList)
    } // Function spFloorAndBuilding

    /**
     * 获取项目下已有的空间类型
     */
    @ApiOperation(value = "获取项目下已有的空间类型", notes = "")
    @PostMapping(value = ["/space-type"])
    fun spaceType(): SQueryResponse<SpaceType>{
        val projectId = SPageContext.getHeader("ProjectId")
        return mepSystemCalc.spaceTypes(projectId!!)
    } // Function

} // Class ZoneAirConditioningController