package com.persagy.server.services.rel

import com.persagy.server.services.base.RService
import com.persagy.server.datacenter.models.entities.rel.RBdForEqList
import com.persagy.server.datacenter.models.entities.rel.REqForBd
import com.persagy.server.datacenter.models.entities.rel.REqForBdList
import com.sybotan.base.extensions.toJson
import com.sybotan.mybatis.SMybatisDao
import com.sybotan.service.SPageContext
import com.sybotan.service.models.enums.SResponseType
import com.sybotan.service.models.responses.SBaseResponse
import com.sybotan.service.models.responses.SCreateResponse
import org.json.JSONObject

/**
 * 设备服务于建筑关系服务
 *
 * @author 张维新
 */
object REqForBdService: RService<REqForBd>(SMybatisDao(REqForBd::class.java)) {


    /**
     * 创建设备服务于建筑关系
     *
     * @param   rEqForBd    设备服务于建筑关系对象
     */
    fun create(rEqForBd: REqForBd): SCreateResponse<REqForBd> {
        return try {
            val projectId = SPageContext.getHeader("ProjectId")
            rEqForBd.projectId = projectId
            val isInsert = insert(rEqForBd)
            if (isInsert){
                val jsonObject = JSONObject()
                jsonObject.put("type", "11")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rEqForBd.equipId)
                jsonObject.put("obj_type", "Eq2Bd")
                jsonObject.put("linkId",rEqForBd.buildingId)

                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())

                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } //Function create()

    /**
     * 根据对象删除
     *
     * @param    rEqForBd     关系对象实例
     * @return
     */
    fun deletes(rEqForBd: REqForBd): SBaseResponse {
        return try {
            val projectId = SPageContext.getHeader("ProjectId")
            val isDelete = delete(rEqForBd)
            /** 删除成功 */
            if (isDelete) {
                val jsonObject = JSONObject()
                jsonObject.put("type", "12")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rEqForBd.equipId)
                jsonObject.put("obj_type", "Eq2Bd")
                jsonObject.put("linkId",rEqForBd.buildingId)

                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())

                val sBaseResponse = SBaseResponse(SResponseType.success)
                sBaseResponse.message = rEqForBd.toJson()
                sBaseResponse
            } else {
                SBaseResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建设备服务于建筑关系
     *
     * @param    rEqForBdList     设备和建筑关系的对象列表
     */
    fun createEq(rEqForBdList: REqForBdList): SCreateResponse<REqForBd> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("equipID = '${rEqForBdList.equipId}'")
            val projectId = SPageContext.getHeader("ProjectId")
            for (buildingId in rEqForBdList.buildingIdList!!){
                val rEqForBd = REqForBd()
                rEqForBd.equipId = rEqForBdList.equipId
                rEqForBd.buildingId = buildingId
                rEqForBd.projectId = projectId
                val replace = replace(rEqForBd)
                if (replace){
                    successList.add(rEqForBd)
                }else{
                    failureList.add(rEqForBd)
                }
            }
            if (successList.size>0){
                val jsonObject = JSONObject()
                jsonObject.put("type", "13")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rEqForBdList.equipId)
                jsonObject.put("obj_type", "Eq2Bd")
                jsonObject.put("linkId",successList.toJson())
                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())

                val sCreateResponse = SCreateResponse<REqForBd>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createEq()

    /**
     * 创建服务于建筑的设备关系
     *
     * @param    rEqForBdList    设备和建筑关系的对象列表
     */
    fun createsBd(rEqForBdList: RBdForEqList): SCreateResponse<REqForBd> {
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("buildingID = '${rEqForBdList.buildingId}'")
            val projectId = SPageContext.getHeader("ProjectId")
            for (equipId in rEqForBdList.equipIdList!!){
                val rEqForBd = REqForBd()
                rEqForBd.buildingId = rEqForBdList.buildingId
                rEqForBd.equipId = equipId
                rEqForBd.projectId = projectId
                val replace = replace(rEqForBd)
                if (replace){
                    successList.add(rEqForBd)
                }else{
                    failureList.add(rEqForBd)
                }
            }
            if (successList.size>0){
                val jsonObject = JSONObject()
                jsonObject.put("type", "13")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rEqForBdList.buildingId)
                jsonObject.put("obj_type", "Eq2Bd")
                jsonObject.put("linkId",successList.toJson())
                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())

                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<REqForBd>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<REqForBd>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsBd()

} // Class REqForBdService