package com.persagy.server.services.rel

import com.persagy.server.services.base.RService
import com.persagy.server.datacenter.models.entities.rel.REqInSh
import com.persagy.server.datacenter.models.entities.rel.REqInShList
import com.persagy.server.datacenter.models.entities.rel.RShInEqList
import com.sybotan.base.extensions.toJson
import com.sybotan.mybatis.SMybatisDao
import com.sybotan.service.SPageContext
import com.sybotan.service.models.enums.SResponseType
import com.sybotan.service.models.responses.SBaseResponse
import com.sybotan.service.models.responses.SCreateResponse
import org.json.JSONObject

/**
 * 设备所在竖井关系服务
 *
 * @author 付国宝
 */
object REqInShService: RService<REqInSh>(SMybatisDao(REqInSh::class.java)) {

    /**
     * 创建设备所在竖井关系
     *
     * @param    rEqInSh     设备所在竖井关系对象
     */
    fun create(rEqInSh: REqInSh): SCreateResponse<REqInSh>{
        return try {
            val projectId = SPageContext.getHeader("ProjectId")
            rEqInSh.projectId = projectId
            val isInsert = insert(rEqInSh)
            if (isInsert){
                val jsonObject = JSONObject()
                jsonObject.put("type", "11")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rEqInSh.equipId)
                jsonObject.put("obj_type", "EquipinShaft")
                jsonObject.put("linkId",rEqInSh.shaftId)

                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "EquipinShaft", "11", IdUtils.uuidCreate(),  rEqInSh.equipId!!,
//                        rEqInSh.shaftId!!),false)


                SCreateResponse(SResponseType.success)
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function create()

    /**
     * 根据对象删除
     *
     * @param     rEqInSh      关系对象实例
     * @return
     */
    fun deletes(rEqInSh: REqInSh): SBaseResponse{
        return try {
            val projectId = SPageContext.getHeader("ProjectId")
            val isDelete = delete(rEqInSh)
            /** 删除成功 */
            if (isDelete) {
                val jsonObject = JSONObject()
                jsonObject.put("type", "12")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rEqInSh.equipId)
                jsonObject.put("obj_type", "EquipinShaft")
                jsonObject.put("linkId",rEqInSh.shaftId)

                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "EquipinShaft", "12", IdUtils.uuidCreate(),  rEqInSh.equipId!!,
//                        rEqInSh.shaftId!!),false)

                val sBaseResponse = SBaseResponse(SResponseType.success)
                sBaseResponse.message = rEqInSh.toJson()
                sBaseResponse
            } else {
                SBaseResponse(SResponseType.failure)
            }
        }catch (e: Exception){
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建设备所在竖井关系
     *
     * @param    rEqInShList     设备和竖井关系的对象列表
     */
    fun createEq(rEqInShList: REqInShList): SCreateResponse<REqInSh>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("equipID = '${rEqInShList.equipId}'")
            val projectId = SPageContext.getHeader("ProjectId")
            for (shaftId in rEqInShList.shaftIdList!!){
                val rEqInSh = REqInSh()
                rEqInSh.equipId = rEqInShList.equipId
                rEqInSh.shaftId = shaftId
                rEqInSh.projectId = projectId
                val replace = replace(rEqInSh)
                if (replace){
                    successList.add(rEqInSh)
                }else{
                    failureList.add(rEqInSh)
                }
            }
            if (successList.size>0){
                val jsonObject = JSONObject()
                jsonObject.put("type", "13")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rEqInShList.equipId)
                jsonObject.put("obj_type", "EquipinShaft")
                jsonObject.put("linkId",successList.toJson())
                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "EquipinShaft", "13", IdUtils.uuidCreate(),  rEqInShList.equipId!!,null),false)
                val sCreateResponse = SCreateResponse<REqInSh>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createEq()

    /**
     * 创建所在竖井的设备关系
     *
     * @param    rShInEqList     设备和竖井关系的对象列表
     */
    fun createsSh(rShInEqList: RShInEqList): SCreateResponse<REqInSh>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("shaftID = '${rShInEqList.shaftId}'")
            val projectId = SPageContext.getHeader("ProjectId")
            for (equipId in rShInEqList.equipIdList!!){
                val rEqInSh = REqInSh()
                rEqInSh.shaftId = rShInEqList.shaftId
                rEqInSh.equipId = equipId
                rEqInSh.projectId = projectId
                val replace = replace(rEqInSh)
                if (replace){
                    successList.add(rEqInSh)
                }else{
                    failureList.add(rEqInSh)
                }
            }
            if (successList.size>0){
                val jsonObject = JSONObject()
                jsonObject.put("type", "13")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rShInEqList.shaftId)
                jsonObject.put("obj_type", "EquipinShaft")
                jsonObject.put("linkId",successList.toJson())
                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "EquipinShaft", "13", IdUtils.uuidCreate(),  rShInEqList.shaftId!!,null),false)
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<REqInSh>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<REqInSh>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsSh()

    /**
     * 创建所在竖井的设备关系
     *
     * @param    rShInEqList     设备和竖井关系的对象列表
     */
    fun createsShEq(rShInEqList: RShInEqList): SCreateResponse<REqInSh>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
//            delete("shaftID = '${rShInEqList.shaftId}'")
            val projectId = SPageContext.getHeader("ProjectId")
            for (equipId in rShInEqList.equipIdList!!){
                val rEqInSh = REqInSh()
                rEqInSh.shaftId = rShInEqList.shaftId
                rEqInSh.equipId = equipId
                rEqInSh.projectId = projectId
                val replace = replace(rEqInSh)
                if (replace){
                    successList.add(rEqInSh)
                }else{
                    failureList.add(rEqInSh)
                }
            }
            if (successList.size>0){
                val jsonObject = JSONObject()
                jsonObject.put("type", "14")
                jsonObject.put("projectId", projectId)
                jsonObject.put("id", rShInEqList.shaftId)
                jsonObject.put("obj_type", "EquipinShaft")
                jsonObject.put("linkId",successList.toJson())

                /** 发送消息 */
                messageHandler.sendMessage(jsonObject.toString())
//                kafkaService.sendNotice("datacenterrel", RelationNotice(projectId!!, "EquipinShaft", "14", IdUtils.uuidCreate(),  rShInEqList.shaftId!!,null),false)
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<REqInSh>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<REqInSh>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsSh()

} // Object REqInShService