package com.persagy.server.services.rel

import com.persagy.server.datacenter.models.entities.rel.RSpinBd
import com.sybotan.database.SFilter
import com.sybotan.mybatis.SMybatisDao
import com.sybotan.service.SObjectService
import com.sybotan.service.SPageContext
import com.sybotan.service.models.enums.SResponseType
import com.sybotan.service.models.requests.SCreateRequest
import com.sybotan.service.models.requests.SUpdateRequest
import com.sybotan.service.models.responses.SBaseResponse
import com.sybotan.service.models.responses.SCreateResponse

/**
 * 业务空间所在的建筑
 *
 * @author 张维新
 */
object RSpInBdService: SObjectService<RSpinBd>(SMybatisDao(RSpinBd::class.java)) {

    /**
     * 创建业务空间和建筑的关系
     */
    fun  spInBdCreates(rSiInSpBase: SCreateRequest<RSpinBd>): SCreateResponse<RSpinBd> {
        try {
            val projectId = SPageContext.getHeader("ProjectId")
            val contentList = rSiInSpBase.content
            if (contentList.isNotEmpty()){
                for (rSpinBd in contentList){

                    if (!rSpinBd.objectType.isNullOrEmpty()){
                        rSpinBd.projectId = projectId
                        val insert = insert(rSpinBd)
                        if (insert){
                            /** 关系创建成功 */

                        }
                    }else{
                        val sCreateResponse = SCreateResponse<RSpinBd>(SResponseType.failure)
                        sCreateResponse.message = "空间类型不能为空"
                        return  sCreateResponse
                    }

                }
                val sCreateResponse = SCreateResponse<RSpinBd>(SResponseType.success)
                return sCreateResponse
            }else{
                val sCreateResponse = SCreateResponse<RSpinBd>(SResponseType.failure)
                sCreateResponse.message = "列表不可以为空"
                return sCreateResponse
            }

        } catch (e: Exception) {
            e.printStackTrace()
            return SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function spInBdCreates()


    /**
     * 删除业务空间也建筑的关系
     */
    fun spInBdDeLs(rSpinBdList: ArrayList<RSpinBd>): SBaseResponse {
        try {
            val projectId = SPageContext.getHeader("ProjectId")
            if (!rSpinBdList.isNullOrEmpty()){
                for (rSpinBd in rSpinBdList){
                    if (!rSpinBd.spaceId.isNullOrEmpty()&&!rSpinBd.buildingId.isNullOrEmpty()){
                        rSpinBd.projectId = projectId
                        val delete = delete(rSpinBd)
                        if (delete){
                            /** 删除成功 */
                        }
                    }else{
                        val sBaseResponse = SBaseResponse(SResponseType.failure)
                        sBaseResponse.message = "spaceId和buildingId 不可以为空"
                        return sBaseResponse
                    }
                }
                return SBaseResponse(SResponseType.success)
            }else{
                val sBaseResponse = SBaseResponse(SResponseType.failure)
                sBaseResponse.message = "列表不能为空"
                return sBaseResponse
            }
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function spInBdDeLs()

    /**
     * 修改业务空间所在的建筑
     */
    fun spInBdUpdates(request: SUpdateRequest<RSpinBd>): SBaseResponse{
        try {
            val projectId = SPageContext.getHeader("ProjectId")
            val contentList = request.content
            if (!contentList.isNullOrEmpty()){

                for (content in contentList){
                    if (!content.buildingId.isNullOrEmpty()&&!content.spaceId.isNullOrEmpty()&&!content.objectType.isNullOrEmpty()){

                        try {
                            val delete = delete(SFilter.eq("projectId", projectId!!), SFilter.eq("SpaceId", content.spaceId!!))
                            if (delete){
                                /** 删除原始的空间和建筑的数据 */

                            }
                        } catch (e: Exception) {
                            e.printStackTrace()
                        }

                        val insert = insert(content)
                        if (insert){
                            /** 添加关系 */
                        }

                    }else{
                        val sBaseResponse = SBaseResponse(SResponseType.failure)
                        sBaseResponse.message = "buildingId、spaceId、objectType 不能为空"
                        return sBaseResponse
                    }
                }
                return SBaseResponse(SResponseType.success)
            }else{
                val sBaseResponse = SBaseResponse(SResponseType.failure)
                sBaseResponse.message = "列表不能为空"
                return sBaseResponse
            }

        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message.toString())
        }

    } // Function spInBdUpdates()

} // Object RSpInBdService