package com.persagy.server.services.rel

import com.persagy.server.datacenter.models.entities.rel.RSpatialConnection
import com.persagy.server.services.objects.ZoneSpaceBaseService
import com.sybotan.database.SFilter
import com.sybotan.mybatis.SMybatisDao
import com.sybotan.service.SObjectService
import com.sybotan.service.SPageContext
import com.sybotan.service.models.enums.SResponseType
import com.sybotan.service.models.requests.SCreateRequest
import com.sybotan.service.models.requests.SQueryRequest
import com.sybotan.service.models.responses.SBaseResponse
import com.sybotan.service.models.responses.SCreateResponse
import com.sybotan.service.models.responses.SQueryResponse
import org.slf4j.LoggerFactory

/**
 * 空间关系
 *
 * @author 张维新
 */
object RSpatialConnectionService : SObjectService<RSpatialConnection>(SMybatisDao(RSpatialConnection::class.java)) {

    private val logger = LoggerFactory.getLogger(RSpatialConnectionService::class.java)

    /**
     * 创建
     */
    fun create(rSiInSpBase: SCreateRequest<RSpatialConnection>): SCreateResponse<RSpatialConnection> {
        return try {
            val projectId = SPageContext.getHeader("ProjectId")
            val contentList = rSiInSpBase.content
            if (contentList.size > 0) {
                for (content in contentList) {
                    content.projectId = projectId!!
                        logger.debug("项目id =${content.projectId}")
                    if (content.graphType.isNullOrEmpty() || content.floorId.isNullOrEmpty()) {
                        val sCreateResponse = SCreateResponse<RSpatialConnection>(SResponseType.failure)
                        sCreateResponse.message = "请检查数据的完整性"
                        return sCreateResponse
                    }

                    if (content.zoneType.isNullOrEmpty()) {
                        val zoneSpace = ZoneSpaceBaseService.select(SFilter.eq("id", content.spaceIdOne!!)).entity()
                        if (zoneSpace != null) {
                            content.zoneType = zoneSpace.objectType
                        }
                    }
                    val insert = replace(content)
                    if (insert) {
                    }
                }
                val sCreateResponse = SCreateResponse<RSpatialConnection>(SResponseType.success)
                sCreateResponse.entityList = contentList
                return sCreateResponse
            } else {
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure, e.message.toString())
        }


    } // Fun onCreateBefore()

    /**
     * 查询
     */
    fun pageQuerys(request: SQueryRequest): SQueryResponse<RSpatialConnection> {
        return try {
            val projectId = SPageContext.getHeader("ProjectId")
            val filter = select(request.filters).filter(SFilter.eq("projectId", projectId!!)).exec()
            val sQueryResponse = SQueryResponse<RSpatialConnection>(SResponseType.success)
            sQueryResponse.total = filter.size.toLong()
            sQueryResponse.content = filter
            sQueryResponse
        } catch (e: Exception) {
            e.printStackTrace()
            SQueryResponse(SResponseType.failure)
        }
    } //

    /**
     * 删除
     */
    fun  delete(idList: ArrayList<RSpatialConnection>): SBaseResponse {
        return try {
            if (idList.size>0){
                val projectId = SPageContext.getHeader("ProjectId")
                for (rSpatialConnection in idList){
                    try {
                        delete(SFilter.eq("zoneType",rSpatialConnection.zoneType!!),SFilter.eq("graphType",rSpatialConnection.graphType!!),SFilter.eq("projectId",projectId!!),SFilter.eq("spaceIdOne",rSpatialConnection.spaceIdOne!!),SFilter.eq("spaceIdTwo",rSpatialConnection.spaceIdTwo!!))
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }

                    try {
                        delete(SFilter.eq("zoneType",rSpatialConnection.zoneType!!),SFilter.eq("graphType",rSpatialConnection.graphType!!),SFilter.eq("projectId",projectId!!),SFilter.eq("spaceIdOne",rSpatialConnection.spaceIdTwo!!),SFilter.eq("spaceIdTwo",rSpatialConnection.spaceIdOne!!))
                    } catch (e: Exception) {
                        e.printStackTrace()
                    }
                }
                SBaseResponse(SResponseType.success)
            }else{
                SBaseResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SBaseResponse(SResponseType.failure)
        }

    } //


    /**
     * 删除某个楼层下的数据
     */
    fun  deleteFloor(rSpatialConnection: RSpatialConnection): SBaseResponse {
        return try {
            val projectId = SPageContext.getHeader("ProjectId")
            delete(SFilter.eq("graphType",rSpatialConnection.graphType!!),SFilter.eq("projectId",projectId!!),SFilter.eq("floorId",rSpatialConnection.floorId!!),SFilter.eq("zoneType",rSpatialConnection.zoneType!!))

            SBaseResponse(SResponseType.success)

        } catch (e: Exception) {
            e.printStackTrace()
            SBaseResponse(SResponseType.failure)
        }

    } //



} // Object RSpatialConnectionService