package com.persagy.server.services.rel

import com.persagy.server.services.base.RService
import com.persagy.server.datacenter.models.entities.rel.RBdInSyList
import com.persagy.server.datacenter.models.entities.rel.RSyInBd
import com.persagy.server.datacenter.models.entities.rel.RSyInBdList
import com.sybotan.base.extensions.toJson
import com.sybotan.mybatis.SMybatisDao
import com.sybotan.service.SPageContext
import com.sybotan.service.models.enums.SResponseType
import com.sybotan.service.models.responses.SBaseResponse
import com.sybotan.service.models.responses.SCreateResponse

/**
 * 系统所在建筑关系服务
 *
 * @author 付国宝
 */
object RSyInBdService: RService<RSyInBd>(SMybatisDao(RSyInBd::class.java)) {

    /**
     * 创建系统所在建筑关系
     *
     * @param    rSyInBd    系统所在建筑关系对象
     */
    fun create(rSyInBd: RSyInBd): SCreateResponse<RSyInBd> {
        return try {
            val projectId = SPageContext.getHeader("ProjectId")
            rSyInBd.projectId = projectId
            val isInsert = insert(rSyInBd)
            if (isInsert){
                SCreateResponse(SResponseType.success)

            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function create()

    /**
     * 根据对象删除
     *
     * @param    rSyInBd    关系对象实例
     * @return
     */
    fun deletes(rSyInBd: RSyInBd): SBaseResponse {
        return try {
            val isDelete = delete(rSyInBd)
            /** 删除成功 */
            if (isDelete) {
                val sBaseResponse = SBaseResponse(SResponseType.success)
                sBaseResponse.message = rSyInBd.toJson()
                sBaseResponse
            } else {
                SBaseResponse(SResponseType.failure)
            }
        }catch (e: Exception){
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure, e.message!!)
        }
    } // Function deletes()

    /**
     * 创建系统所在建筑关系
     *
     * @param    rSyInBdList    系统和建筑关系的对象列表
     */
    fun createSy(rSyInBdList: RSyInBdList): SCreateResponse<RSyInBd>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("sysID = '${rSyInBdList.sysId}'")
            val projectId = SPageContext.getHeader("ProjectId")
            if (rSyInBdList.buildingIdList!=null&&rSyInBdList.buildingIdList!!.size>0){
                for (buildingId in rSyInBdList.buildingIdList!!){
                    val rSyInBd = RSyInBd()
                    rSyInBd.sysId = rSyInBdList.sysId
                    rSyInBd.buildingId = buildingId
                    rSyInBd.projectId = projectId
                    val replace = replace(rSyInBd)
                    if (replace){
                        successList.add(rSyInBd)
                    }else{
                        failureList.add(rSyInBd)
                    }
                }
                if (successList.size>0){
                    val sCreateResponse = SCreateResponse<RSyInBd>(SResponseType.success)
                    if (failureList.size>0){
                        sCreateResponse.message ="失败对象："+failureList.toJson()
                    }
                    sCreateResponse
                }else{
                    SCreateResponse(SResponseType.failure)
                }
            }else{
                SCreateResponse(SResponseType.success)
            }

        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createSy()

    /**
     * 创建系统所在建筑关系
     *
     * @param    rSyInBdList    系统和建筑关系的对象列表
     */
    fun createSyBd(rSyInBdList: RSyInBdList): SCreateResponse<RSyInBd>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
//            delete("sysID = '${rSyInBdList.sysId}'")
            val projectId = SPageContext.getHeader("ProjectId")
            for (buildingId in rSyInBdList.buildingIdList!!){
                val rSyInBd = RSyInBd()
                rSyInBd.sysId = rSyInBdList.sysId
                rSyInBd.buildingId = buildingId
                rSyInBd.projectId = projectId
                val replace = replace(rSyInBd)
                if (replace){
                    successList.add(rSyInBd)
                }else{
                    failureList.add(rSyInBd)
                }
            }
            if (successList.size>0){
                val sCreateResponse = SCreateResponse<RSyInBd>(SResponseType.success)
                if (failureList.size>0){
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                }
                sCreateResponse
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e: Exception) {
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createSy()

    /**
     * 创建所在建筑的系统关系
     *
     * @param    rBdInSyList    系统和建筑关系的对象列表
     */
    fun createsBd(rBdInSyList: RBdInSyList): SCreateResponse<RSyInBd>{
        return try {
            /** 成功集合 */
            val successList = ArrayList<Any>()
            /** 失败集合 */
            val failureList = ArrayList<Any>()
            delete("buildingID = '${rBdInSyList.buildingId}'")
            val projectId = SPageContext.getHeader("ProjectId")
            for (sysId in rBdInSyList.sysIdList!!){
                val rSyInBd = RSyInBd()
                rSyInBd.buildingId = rBdInSyList.buildingId
                rSyInBd.sysId = sysId
                rSyInBd.projectId = projectId
                val replace = replace(rSyInBd)
                if (replace){
                    successList.add(rSyInBd)
                }else{
                    failureList.add(rSyInBd)
                }
            }
            if (successList.size>0){
                if (failureList.size>0){
                    val sCreateResponse = SCreateResponse<RSyInBd>(SResponseType.success)
                    sCreateResponse.message ="失败对象："+failureList.toJson()
                    sCreateResponse
                }else{
                    val sCreateResponse = SCreateResponse<RSyInBd>(SResponseType.success)
                    sCreateResponse
                }
            }else{
                SCreateResponse(SResponseType.failure)
            }
        } catch (e:Exception){
            e.printStackTrace()
            SCreateResponse(SResponseType.failure,e.message.toString())
        }
    } // Function createsBd()

} // Object RSyInBdService