/*
 * ********************************************************************************************************************
 *
 *                     :*$@@%$*:                         ;:                ;;    ;;
 *                   :@@%!  :!@@%:                       %!             ;%%@@%$ =@@@@@@@%;     @%@@@%%%%@@@@@
 *                  :@%;       :$=                       %%$$$%$$         ;$$  ;$@=   !@$
 *                  =@!                                  %!              @ $=;%   !@@@%:      !$$$$$$$$$$$$$$=
 *                  =@*                                  %!              @ $= % %@=   =%@!      %=
 *             *$%%! @@=        ;=$%%%$*:                %!              @ $= % =%%%%%%@$      *%:         =%
 *           %@@!:    !@@@%=$@@@@%!  :*@@$:              %!              @ $= % $*     ;@      @*          :%*
 *         ;@@!          ;!!!;:         ;@%:      =======@%========*     @ $$ % $%*****$@     :@$=*********=@$
 *         $@*   ;@@@%=!:                *@*
 *         =@$    ;;;!=%@@@@=!           =@!
 *          %@$:      =@%: :*@@@*       %@=                    Copyright (c) 2016-2019.  北京上格云技术有限公司
 *           ;%@@$=$@@%*       *@@@$=%@@%;
 *              ::;::             ::;::                                              All rights reserved.
 *
 * ********************************************************************************************************************
 */

package com.persagy.server.services.relation_calc

import cn.sagacloud.server.datacenter.utils.IdUtils
import com.persagy.server.datacenter.models.entities.graphtype.RelationTypeProject
import com.persagy.server.mappers.IRelationCalcMapper
import com.persagy.server.services.assistant.RUpDateEquipAndSapceService
import com.sybotan.database.SFilter
import com.sybotan.mybatis.SMybatisDao
import com.sybotan.service.SBaseService
import com.sybotan.service.models.enums.SResponseType
import com.sybotan.service.models.responses.SBaseResponse
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.stereotype.Service

/**
 * 自动关系计算
 *
 * @author  庞利祥 <sybotan@126.com>
 */
@Service
open class RelationCalcService {
    /** 实例化 mapper 对象 */
    @Autowired
    lateinit var mapper: IRelationCalcMapper

    /** 关系总览服务 */
    val relationTypeProjectservice = SBaseService(SMybatisDao(RelationTypeProject::class.java))
    companion object {
        /** 日志 */
        private val logger = LoggerFactory.getLogger(RelationCalcService::class.java)

    } // Companion object

    /**
     * 建筑下的业务空间
     *
     * @param   projectId   项目id
     * @param   tables      参与计算的表的全名称, 带schema名, 以英文逗号隔开
     */
    fun calcBd2Sp(tables: String? , projectId : String): SBaseResponse {
        try {
            var tablesName = ""
            if(tables == null){
                tablesName = "public.zone_air_conditioning," +
                        "public.zone_clean," +
                        "public.zone_domestic_water_supply," +
                        "public.zone_fire," +
                        "public.zone_function," +
                        "public.zone_general," +
                        "public.zone_heating," +
                        "public.zone_lighting," +
                        "public.zone_network," +
                        "public.zone_power_supply," +
                        "public.zone_security," +
                        "public.zone_tenant"
            } else {
                val nameList = tables.split(",")
                for(type in nameList){
                    when (type) {
                        "HeatingZone" -> {
                            tablesName = tablesName + "public.zone_heating,"
                        }
                        "CleanZone" -> {
                            tablesName = tablesName + "public.zone_clean,"
                        }
                        "DomesticWaterSupplyZone" -> {
                            tablesName = tablesName + "public.zone_domestic_water_supply,"
                        }
                        "NetworkZone" -> {
                            tablesName = tablesName + "public.zone_network,"
                        }
                        "TenantZone" -> {
                            tablesName = tablesName + "public.zone_tenant,"
                        }
                        "AirConditioningZone" -> {
                            tablesName = tablesName + "public.zone_air_conditioning,"
                        }
                        "FunctionZone" -> {
                            tablesName = tablesName + "public.zone_function,"
                        }
                        "FireZone" -> {
                            tablesName = tablesName + "public.zone_fire,"
                        }
                        "SecurityZone" -> {
                            tablesName = tablesName + "public.zone_security,"
                        }
                        "GeneralZone" -> {
                            tablesName = tablesName + "public.zone_general,"
                        }
                        "PowerSupplyZone" -> {
                            tablesName = tablesName + "public.zone_power_supply,"
                        }
                        "LightingZone" -> {
                            tablesName = tablesName + "public.zone_lighting,"
                        }
                        else -> {
                            return SBaseResponse(SResponseType.failure,"空间分区类型不存在：$type")
                        }
                    }
                }
                if(tablesName!=""&&!tablesName.equals("")){
                    tablesName = tablesName.substring(0,tablesName.length-1)
                }
            }
            upDateBiaoji(projectId,"bd2sp",3)
            val result = mapper.calcBd2Sp(tablesName,projectId)
            if(result != null && result == 0){
                upDate(projectId,"bd2sp")
//                upDate(projectId,"fl2sp")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"bd2sp",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcBd2Sp()

    /**
     * 设备所在建筑
     * @param   projectId       项目id
     */
    fun calcEq2Bd(projectId : String): SBaseResponse {
        try {
//            upDateBiaoji(projectId,"bd2sp",3)
            val result = mapper.calcEq2Bd(projectId)
            if(result != null && result == 0){
//                upDate(projectId,"fl2sp")
                return SBaseResponse(SResponseType.success)
            }else{
//                upDateBiaoji(projectId,"bd2sp",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcEq2Bd()

    /**
     * 设备所在楼层
     * @param   projectId       项目id
     */
    fun calcEq2Fl(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"eq2fl",3)
            val result = mapper.calcEq2Fl(projectId)
            if(result != null && result == 0 ){
                upDate(projectId,"eq2fl")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"eq2fl",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcEq2Fl()

    /**
     * 设备所在竖井
     * @param   projectId   项目id
     */
    fun calcEq2Sh(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"eq2sh",3)
            upDateBiaoji(projectId,"eq2fl",3)
            upDateBiaoji(projectId,"fl2sp",3)
            val entity = relationTypeProjectservice.select(SFilter.eq("projectId", projectId), SFilter.eq("relationType", "eq2sp_in")).entity()
            if (entity!=null){
                entity.computationalState = 3
                relationTypeProjectservice.update(entity)
            }
            val result = mapper.calcEq2Sh(projectId)
            if(result != null && result == 0 ){
                upDate(projectId,"eq2sh")
                upDate(projectId,"eq2sp_in")
                upDate(projectId,"fl2sp")
                upDate(projectId,"eq2fl")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"eq2sh",5)
                upDateBiaoji(projectId,"eq2fl",5)
                upDateBiaoji(projectId,"fl2sp",5)
                val entity = relationTypeProjectservice.select(SFilter.eq("projectId", projectId), SFilter.eq("relationType", "eq2sp_in")).entity()
                if (entity!=null){
                    entity.computationalState = 5
                    relationTypeProjectservice.update(entity)
                }
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcEq2Sh()

    /**
     * 设备所在业务空间
     *
     *  @param  tables      参与计算的业务空间(为空表示所有业务空间类型)，以英文逗号隔开 HeatingZone,CleanZone,DomesticWaterSupplyZone,NetworkZone,TenantZone,AirConditioningZone FunctionZone,FireZone,SecurityZone,GeneralZone,PowerSupplyZone,LightingZone
     *  @param  projectId   项目id
     *  @param  sign        是否要删除自动维护的关系（不填默认为false）, boolean型, 值为true或false (true表示删除自动和手动维护的关系, false表示只删除手动计算的的关系)
     */
    fun calcEq2Sp(tables: String?, projectId: String, sign: Boolean): SBaseResponse {
        try {
            var tablesName = ""
            var outTablesName = ""
            var sign1Int: Int = 2
            var sign2Int: Int = 2
            if(tables == null){
                tablesName = "public.zone_air_conditioning," +
                        "public.zone_clean," +
                        "public.zone_domestic_water_supply," +
                        "public.zone_fire," +
                        "public.zone_function," +
                        "public.zone_general," +
                        "public.zone_heating," +
                        "public.zone_lighting," +
                        "public.zone_network," +
                        "public.zone_power_supply," +
                        "public.zone_security," +
                        "public.zone_tenant"
                outTablesName = "public.r_eq_in_sp_zone_air_conditioning," +
                        "public.r_eq_in_sp_zone_clean," +
                        "public.r_eq_in_sp_zone_domestic_water_supply," +
                        "public.r_eq_in_sp_zone_fire," +
                        "public.r_eq_in_sp_zone_function," +
                        "public.r_eq_in_sp_zone_general," +
                        "public.r_eq_in_sp_zone_heating," +
                        "public.r_eq_in_sp_zone_lighting," +
                        "public.r_eq_in_sp_zone_network," +
                        "public.r_eq_in_sp_zone_power_supply," +
                        "public.r_eq_in_sp_zone_security," +
                        "public.r_eq_in_sp_zone_tenant"
            } else {
                val nameList = tables.split(",")
                for(type in nameList){
                    when (type) {
                        "HeatingZone" -> {
                            tablesName = tablesName + "public.zone_heating,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_heating,"
                        }
                        "CleanZone" -> {
                            tablesName = tablesName + "public.zone_clean,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_clean,"
                        }
                        "DomesticWaterSupplyZone" -> {
                            tablesName = tablesName + "public.zone_domestic_water_supply,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_domestic_water_supply,"
                        }
                        "NetworkZone" -> {
                            tablesName = tablesName + "public.zone_network,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_network,"
                        }
                        "TenantZone" -> {
                            tablesName = tablesName + "public.zone_tenant,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_tenant,"
                        }
                        "AirConditioningZone" -> {
                            tablesName = tablesName + "public.zone_air_conditioning,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_air_conditioning,"
                        }
                        "FunctionZone" -> {
                            tablesName = tablesName + "public.zone_function,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_function,"
                        }
                        "FireZone" -> {
                            tablesName = tablesName + "public.zone_fire,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_fire,"
                        }
                        "SecurityZone" -> {
                            tablesName = tablesName + "public.zone_security,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_security,"
                        }
                        "GeneralZone" -> {
                            tablesName = tablesName + "public.zone_general,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_general,"
                        }
                        "PowerSupplyZone" -> {
                            tablesName = tablesName + "public.zone_power_supply,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_power_supply,"
                        }
                        "LightingZone" -> {
                            tablesName = tablesName + "public.zone_lighting,"
                            outTablesName = outTablesName + "public.r_eq_in_sp_zone_lighting,"
                        }
                        else -> {
                            return SBaseResponse(SResponseType.failure,"空间分区类型不存在：$type")
                        }
                    }
                }
                if(tablesName!=""&&!tablesName.equals("")){
                    tablesName = tablesName.substring(0,tablesName.length-1)
                    outTablesName = outTablesName.substring(0,outTablesName.length-1)
                }
            }
            if(sign){
                sign1Int = 1
                sign2Int = 2
            }

            try {
                upDateBiaoji(projectId,"eq2fl",3)
                upDateBiaoji(projectId,"fl2sp",3)

                val entity = relationTypeProjectservice.select(SFilter.eq("projectId", projectId), SFilter.eq("relationType", "eq2sp_in"), SFilter.eq("zoneType",tables!!)).entity()
                if (entity!=null){
                    entity.computationalState = 3
                    relationTypeProjectservice.update(entity)
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            val result = mapper.calcEq2Sp(tablesName,outTablesName,projectId,sign1Int,sign2Int)
            if(result!=null && result == 0){
                try {
                    upDate(projectId,"fl2sp")
                    upDate(projectId,"eq2fl")
                    val entity = relationTypeProjectservice.select(SFilter.eq("projectId", projectId), SFilter.eq("relationType", "eq2sp_in"), SFilter.eq("zoneType",tables!!)).entity()
                    if (entity!=null){
                        entity.computationalState = 1
                        relationTypeProjectservice.update(entity)
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"eq2fl",5)
                upDateBiaoji(projectId,"fl2sp",5)
                val entity = relationTypeProjectservice.select(SFilter.eq("projectId", projectId), SFilter.eq("relationType", "eq2sp_in"), SFilter.eq("zoneType",tables!!)).entity()
                if (entity!=null){
                    entity.computationalState = 5
                    relationTypeProjectservice.update(entity)
                }
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcEq2Sp()

    /**
     * 楼层贯通关系
     *
     * @param   projectId   项目id
     * @param   sign1       表示r_fl_through_fl表中要被删除的sign值
     * @param   sign2       同上 与sign1是逻辑或的关系
     */
    fun calcFl2Fl(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"fl2fl",3)
            val result = mapper.calcFl2Fl(projectId,2,2)
            if(result!=null && result == 0){
                try {
                    val entity = RUpDateEquipAndSapceService.relationTypeProjectservice.select(SFilter.eq("projectId", projectId), SFilter.eq("relationTypeCode", "Fl2Fl")).entity()
                    if (entity!=null){
                        entity.computationalState = 1
                        RUpDateEquipAndSapceService.relationTypeProjectservice.update(entity)
                    }
                } catch (e: Exception) {
                    e.printStackTrace()
                }

                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"fl2fl",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcFl2Fl()

    /**
     * 楼层下的业务空间
     *
     * @param   projectId   项目id
     * @param   tables      参与计算的表的全名称, 带schema名, 以英文逗号隔开
     */
    fun calcFl2Sp(tables: String? , projectId : String): SBaseResponse {
        try {
            var tablesName = ""
            if(tables == null){
                tablesName = "public.zone_air_conditioning," +
                        "public.zone_clean," +
                        "public.zone_domestic_water_supply," +
                        "public.zone_fire," +
                        "public.zone_function," +
                        "public.zone_general," +
                        "public.zone_heating," +
                        "public.zone_lighting," +
                        "public.zone_network," +
                        "public.zone_power_supply," +
                        "public.zone_security," +
                        "public.zone_tenant"
            } else {
                val nameList = tables.split(",")
                for(type in nameList){
                    when (type) {
                        "HeatingZone" -> {
                            tablesName = tablesName + "public.zone_heating,"
                        }
                        "CleanZone" -> {
                            tablesName = tablesName + "public.zone_clean,"
                        }
                        "DomesticWaterSupplyZone" -> {
                            tablesName = tablesName + "public.zone_domestic_water_supply,"
                        }
                        "NetworkZone" -> {
                            tablesName = tablesName + "public.zone_network,"
                        }
                        "TenantZone" -> {
                            tablesName = tablesName + "public.zone_tenant,"
                        }
                        "AirConditioningZone" -> {
                            tablesName = tablesName + "public.zone_air_conditioning,"
                        }
                        "FunctionZone" -> {
                            tablesName = tablesName + "public.zone_function,"
                        }
                        "FireZone" -> {
                            tablesName = tablesName + "public.zone_fire,"
                        }
                        "SecurityZone" -> {
                            tablesName = tablesName + "public.zone_security,"
                        }
                        "GeneralZone" -> {
                            tablesName = tablesName + "public.zone_general,"
                        }
                        "PowerSupplyZone" -> {
                            tablesName = tablesName + "public.zone_power_supply,"
                        }
                        "LightingZone" -> {
                            tablesName = tablesName + "public.zone_lighting,"
                        }
                        else -> {
                            return SBaseResponse(SResponseType.failure,"空间分区类型不存在：$type")
                        }
                    }
                }
                if(tablesName!=""&&!tablesName.equals("")){
                    tablesName = tablesName.substring(0,tablesName.length-1)
                }
            }
            upDateBiaoji(projectId,"fl2sp",3)
            val result = mapper.calcFl2Sp(tablesName,projectId)
            if(result != null && result == 0 ){
                upDate(projectId,"fl2sp")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"fl2sp",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcFl2Sp()

    /**
     * 资产所在建筑
     *
     * @param   projectId        项目id
     */
    fun calcPe2Bd(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"pe2bd",3)
            upDateBiaoji(projectId,"eq2fl",3)
            val result = mapper.calcPe2Bd(projectId)
            if(result != null && result == 0){
                upDate(projectId,"pe2bd")
                upDate(projectId,"eq2fl")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"pe2bd",5)
                upDateBiaoji(projectId,"eq2fl",2)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcPe2Bd()

    /**
     * 资产所在楼层
     *
     * @param   projectId     项目id
     */
    fun calcPe2Fl(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"pe2fl",3)
            upDateBiaoji(projectId,"eq2fl",3)
            val result = mapper.calcPe2Fl(projectId)
            if(result != null && result == 0 ){
                upDate(projectId,"pe2fl")
                upDate(projectId,"eq2fl")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"pe2fl",5)
                upDateBiaoji(projectId,"eq2fl",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcPe2Fl()

    /**
     * 资产所在竖井
     *
     * @param   projectId       项目id
     */
    fun calcPe2Sh(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"pe2sh",3)
            upDateBiaoji(projectId,"fl2sp",3)
            upDateBiaoji(projectId,"eq2sp_in",3)
            upDateBiaoji(projectId,"pe2sp",3)
            upDateBiaoji(projectId,"eq2sh",3)
            upDateBiaoji(projectId,"eq2fl",3)
            val result = mapper.calcPe2Sh(projectId)
            if(result != null && result == 0){
                upDate(projectId,"pe2sh")
                upDate(projectId,"fl2sp")
                upDate(projectId,"eq2sp_in")
                upDate(projectId,"pe2sp")
                upDate(projectId,"eq2sh")
                upDate(projectId,"eq2fl")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"pe2sh",5)
                upDateBiaoji(projectId,"fl2sp",5)
                upDateBiaoji(projectId,"eq2sp_in",5)
                upDateBiaoji(projectId,"pe2sp",5)
                upDateBiaoji(projectId,"eq2sh",5)
                upDateBiaoji(projectId,"eq2fl",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcPe2Sh()

    /**
     * 资产所在业务空间
     *
     * @param   projectId       项目id
     */
    fun calcPe2Sp(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"eq2fl",3)
            upDateBiaoji(projectId,"fl2sp",3)
            upDateBiaoji(projectId,"eq2sp_in",3)
            upDateBiaoji(projectId,"pe2sp",3)
            val result = mapper.calcPe2Sp(projectId)
            if(result != null && result == 0){
                upDate(projectId,"pe2sp")
                upDate(projectId,"eq2fl")
                upDate(projectId,"fl2sp")
                upDate(projectId,"eq2sp_in")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"pe2sp",5)
                upDateBiaoji(projectId,"eq2fl",5)
                upDateBiaoji(projectId,"fl2sp",5)
                upDateBiaoji(projectId,"eq2sp_in",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcPe2Sp()

    /**
     * 建筑下的竖井
     *
     * @param   projectId       项目id
     */
    fun calcSh2Bd(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"sh2bd",3)
            val result = mapper.calcSh2Bd(projectId)
            if(result != null && result == 0){
                upDate(projectId,"sh2bd")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"sh2bd",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcSh2Bd()

    /**
     * 竖井贯通
     *
     * @param   projectId       项目id
     */
    fun calcSh2Sh(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"sh2sh",3)
            val result = mapper.calcSh2Sh(projectId)
            if(result != null && result == 0){
                upDate(projectId,"sh2sh")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"sh2sh",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcPe2Sp()

    /**
     * 建筑下的系统设备部件
     *
     * @param   projectId       项目id
     */
    fun calcSyEq2Bd(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"syeq2bd",3)
            upDateBiaoji(projectId,"eq2fl",3)
            val resultEq = mapper.calcEq2Bd(projectId)
            val resultSy = mapper.calcSy2Bd(projectId)

            logger.debug("设备= ${ resultEq}")
            logger.debug("系统= ${ resultSy}")
            if(resultEq != null && resultSy!=null && resultEq == 0 && resultSy == 0 ){
                upDate(projectId,"syeq2bd")
                upDate(projectId,"eq2fl")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"syeq2bd",5)
                upDateBiaoji(projectId,"eq2fl",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcSyEq2Bd()

    /**
     * 系统所在建筑
     *
     * @param   projectId       项目id
     */
    fun calcSy2Bd(projectId : String): SBaseResponse {
        try {
            val result = mapper.calcSy2Bd(projectId)
            if(result != null && result == 0){
//                upDate(projectId,"sh2sh")
                return SBaseResponse(SResponseType.success)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcSy2Bd()

    /**
     * 系统所在楼层
     *
     * @param   projectId       项目id
     */
    fun calcSy2Fl(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"sy2fl",3)
            upDateBiaoji(projectId,"eq2fl",3)
            val result = mapper.calcSy2Fl(projectId)
            if(result != null && result == 0){
                upDate(projectId,"sy2fl")
                upDate(projectId,"eq2fl")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"sy2fl",5)
                upDateBiaoji(projectId,"eq2fl",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcSy2Fl()

    /**
     * 系统所在竖井
     *
     * @param   projectId       项目id
     */
    fun calcSy2Sh(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"sy2sh",3)
            upDateBiaoji(projectId,"eq2sh",3)
            upDateBiaoji(projectId,"eq2fl",3)
            upDateBiaoji(projectId,"fl2sp",3)
            upDateBiaoji(projectId,"eq2sp_in",3)
            val result = mapper.calcSy2Sh(projectId)
            if(result != null && result == 0){
                upDate(projectId,"sy2sh")
                upDate(projectId,"eq2sh")
                upDate(projectId,"eq2fl")
                upDate(projectId,"fl2sp")
                upDate(projectId,"eq2sp_in")
                return SBaseResponse(SResponseType.success)
            }else {
                upDateBiaoji(projectId,"sy2sh",5)
                upDateBiaoji(projectId,"eq2sh",5)
                upDateBiaoji(projectId,"eq2fl",5)
                upDateBiaoji(projectId,"fl2sp",5)
                upDateBiaoji(projectId,"eq2sp_in",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcSy2Sh()

    /**
     * 系统所在业务空间
     *
     * @param   projectId       项目id
     */
    fun calcSy2Sp(projectId : String): SBaseResponse {
        try {
            upDateBiaoji(projectId,"eq2fl",3)
            upDateBiaoji(projectId,"fl2sp",3)
            upDateBiaoji(projectId,"sy2sp",3)
            upDateBiaoji(projectId,"eq2sp_in",3)
            val result = mapper.calcSy2Sp(projectId)
            if(result != null && result == 0){
                upDate(projectId,"sy2sp")
                upDate(projectId,"fl2sp")
                upDate(projectId,"sy2sp")
                upDate(projectId,"eq2sp_in")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"sy2sp",5)
                upDateBiaoji(projectId,"eq2fl",5)
                upDateBiaoji(projectId,"fl2sp",5)
                upDateBiaoji(projectId,"eq2sp_in",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Function calcSy2Sp()


    fun sp2spTransport(projectId : String): SBaseResponse{
        try {
            upDateBiaoji(projectId,"sp2sp_TrafficNetwork_1",3)
        val result = mapper.calcsp2spTransport(projectId)
            if(result != null && result == 0){

                upDate(projectId,"sp2sp_TrafficNetwork_1")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"sp2sp_TrafficNetwork_1",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Fun

    fun sp2spLight(projectId : String): SBaseResponse{
        try {
            upDateBiaoji(projectId,"sp2sp_RadiationNetwork_1",3)
            val result = mapper.calcsp2spLight(projectId)
            if(result != null && result == 0){

                upDate(projectId,"sp2sp_RadiationNetwork_1")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"sp2sp_RadiationNetwork_1",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    } // Fun

    fun sp2spConvection(projectId: String): SBaseResponse{
        try {
        upDateBiaoji(projectId,"sp2sp_ConvectionNetwork_1",3)
        val result = mapper.calcsp2spConvection(projectId)
        if(result != null && result == 0){

            upDate(projectId,"sp2sp_ConvectionNetwork_1")
            return SBaseResponse(SResponseType.success)
        }else{
            upDateBiaoji(projectId,"sp2sp_ConvectionNetwork_1",5)
        }
        return SBaseResponse(SResponseType.failure)
    } catch (e: Exception) {
        e.printStackTrace()
        return SBaseResponse(SResponseType.failure,e.message!!)
    }
    }

    fun sp2spV2(projectId: String): SBaseResponse{
        try {
            upDateBiaoji(projectId,"sp2sp_SpaceNeighborhood_5",3)
            val result = mapper.calcsp2spV2(projectId)
            if(result != null && result == 0){

                upDate(projectId,"sp2sp_SpaceNeighborhood_5")
                return SBaseResponse(SResponseType.success)
            }else{
                upDateBiaoji(projectId,"sp2sp_SpaceNeighborhood_5",5)
            }
            return SBaseResponse(SResponseType.failure)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }
    }




    /**
     * 修改标记
     */
    fun upDate(projectId: String,type: String){
        try {

            val entity = relationTypeProjectservice.select(SFilter.eq("projectId", projectId), SFilter.eq("relationType", type)).entity()
            if (entity!=null){
                entity.computationalState = 1
                entity.computingTime = IdUtils.time()
                RUpDateEquipAndSapceService.relationTypeProjectservice.update(entity)
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    } // Fun

    /**
     * 计算中标记
     */
    fun upDateBiaoji(projectId: String,type: String,sing: Int){
        try {
            val list = ArrayList<String>()
            list.add("ComputationalState")
            list.add("ComputingTime")
                if (sing == 3){
                    logger.debug("type1= $type********************************************")
                    val entity = relationTypeProjectservice.select(SFilter.eq("projectId", projectId), SFilter.eq("relationType", type)).entity()
                    if (entity!=null){
                        entity.computationalState = 3
                        entity.computingTime = IdUtils.time()
                        logger.debug("computingTime= ${entity.computingTime},graphic_id =${entity.graphicId} ,code = ${entity.relationTypeCode},id= ${entity.id},project_id = ${entity.projectId}********************************************")
                        val update = relationTypeProjectservice.update(entity, list)
                        logger.debug("update= ${update}********************************************")
                    }
                    logger.debug("type= $type********************************************")
                }else{
                    val entity = relationTypeProjectservice.select(SFilter.eq("projectId", projectId), SFilter.eq("relationType", type)).entity()
                    if (entity!=null){
                        entity.computationalState = 5
                        entity.computingTime = IdUtils.time()
                        relationTypeProjectservice.update(entity,list)
                    }
                    logger.debug("type3= $type********************************************")
                }
            logger.debug("type2= $type********************************************")
        } catch (e: Exception) {
            e.printStackTrace()
        }
    } // Fun

    /**
     * 设置标记
     */
    fun setMarks(projectId: String): SBaseResponse{
        try {
            /** 模型重新上传做标记 */
                    val list = ArrayList<String>()
                    list.add("eq2fl")
                    list.add("fl2sp")
                    list.add("bd2sp")
                    list.add("eq2sp_in")
                    for (type in list){
                        val entityList = RUpDateEquipAndSapceService.relationTypeProjectservice.select(SFilter.eq("projectId", projectId), SFilter.eq("relationType", type)).exec()
                        if (entityList.size>0){
                            for (entity in entityList){
                                entity.computationalState = 2
                                RUpDateEquipAndSapceService.relationTypeProjectservice.update(entity)
                            }
                        }
                    }
            return SBaseResponse(SResponseType.success)
        } catch (e: Exception) {
            e.printStackTrace()
            return SBaseResponse(SResponseType.failure,e.message!!)
        }

    } // Fun

} // Class RelationCalcService